%define kversion   4.1.0-rc6
%define rpmversion 4.1.rc6

Name: perf
Summary: Performance monitoring for the Linux kernel
Group: Development/System
License: GPLv2
URL: http://www.kernel.org/
Version: %{rpmversion}
Release: 3%{?dist}
BuildRequires: patch bash sh-utils tar
BuildRequires: bzip2 xz findutils gzip make
BuildRequires: gcc binutils bc bison flex asciidoc xmlto
BuildRequires: elfutils-devel zlib-devel binutils-devel python-devel perl(ExtUtils::Embed)
BuildRequires: audit-libs-devel numactl-devel

# perf tar file created using 'make perf-targz-src-pkg' for
# specific kernel version
Source0: perf-%{kversion}.tar.gz

# Patches on top of perf from Linus' tree (or some other kernel tree) 
#
# These patches come from David Ahern and add the following features:
# - 'perf sched timehist' command
# - options to show time as time-of-day versus perf_clock
# - analysis option to specify a time window of interest
# - perf-trace option to display events for specific task
#
# Going forward if the patches fail to apply you can:
# 1. drop them and lose the features
# 2. port them
# 3. check for updates: https://github.com/dsahern/linux

Patch01: 0001-perf-sched-Introduce-timehist-command.patch
Patch02: 0002-perf-sched-Add-documentation-for-timehist-options.patch
Patch03: 0003-perf-sched-timehist-Add-support-for-context-switch-e.patch
Patch04: 0004-perf-sched-timehist-Add-pstree-option.patch
Patch05: 0005-perf-timehist-Handle-cpu-migration-events.patch
Patch06: 0006-perf-header-move-perf_header__getbuffer64-up-in-head.patch
Patch07: 0007-perf-add-reference-time-as-feature-in-header.patch
Patch08: 0008-perf-tool-Add-time-based-utility-functions.patch
Patch09: 0009-perf-tool-Add-method-for-creating-perf_clock-to-tod-.patch
Patch10: 0010-perf-record-Add-time-of-day-option.patch
Patch11: 0011-perf-script-add-time-of-day-option-for-displaying-ev.patch
Patch12: 0012-perf-sched-timehist-Use-time-of-day-for-timestamps-i.patch
Patch13: 0013-perf-trace-Add-option-to-show-time-of-day-in-output.patch
Patch14: 0014-perf-tool-Add-functions-to-specify-time-window-of-in.patch
Patch15: 0015-perf-sched-timehist-Add-option-to-specify-time-windo.patch
Patch16: 0016-perf-script-Add-option-to-specify-time-window-of-int.patch
Patch17: 0017-perf-report-Add-option-to-specify-time-window-of-int.patch
Patch18: 0018-Add-module-to-export-perf_clock-to-userspace.patch
Patch19: 0019-perf-record-Allow-user-to-control-kernel-user-callch.patch
Patch20: 0020-perf-trace-Update-tid-pid-filtering-option-to-levera.patch
Patch21: 0021-perf-trace-Add-option-to-display-events-only-for-giv.patch
Patch22: 0022-perf-sched-timehist-Add-support-for-hard-and-softirq.patch
Patch23: 0023-perf-sched-timehist-Add-support-for-filtering-on-CPU.patch
Patch24: 0024-perf-sched-timehist-Cleanup-migrations-handling.patch

# patches to make perf work better with sparc
Patch25: 0025-perf-record-Wakeup-based-on-100k-watermark-WIP.patch
Patch26: 0026-perf-sparc-Disable-use-of-kcore.patch
Patch27: 0027-perf-sparc-Slow-down-closing-of-fd-s.patch

# patch to work round the uek2 bus error issue
Patch28: 0028-perf-uek2-bus-error-work-round.patch
Patch29: 0029-perf-uek2-bus-error-work-round-buffer-read.patch

# Cherry pick upstream 	2b42b09b88c8 orabug 22175225
Patch30: 0030-perf-fix_to_show_proper_convergence_stats.patch

%description
This package contains the perf tool, which enables performance monitoring
of the Linux kernel.

# the following is needed to generate the debuginfo rpm
%debug_package

%prep
patch_command='patch -p1 -F1 -s'
ApplyPatch()
{
  local patch=$RPM_SOURCE_DIR/$1
  shift
  if [ ! -f $patch ]; then
    exit 1
  fi
  $patch_command ${1+"$@"} < $patch
}

rm -rf perf-%{kversion}
tar -zxf %{SOURCE0}
echo perf-%{kversion}
cd perf-%{kversion}

ApplyPatch 0001-perf-sched-Introduce-timehist-command.patch
ApplyPatch 0002-perf-sched-Add-documentation-for-timehist-options.patch
ApplyPatch 0003-perf-sched-timehist-Add-support-for-context-switch-e.patch
ApplyPatch 0004-perf-sched-timehist-Add-pstree-option.patch
ApplyPatch 0005-perf-timehist-Handle-cpu-migration-events.patch
ApplyPatch 0006-perf-header-move-perf_header__getbuffer64-up-in-head.patch
ApplyPatch 0007-perf-add-reference-time-as-feature-in-header.patch
ApplyPatch 0008-perf-tool-Add-time-based-utility-functions.patch
ApplyPatch 0009-perf-tool-Add-method-for-creating-perf_clock-to-tod-.patch
ApplyPatch 0010-perf-record-Add-time-of-day-option.patch
ApplyPatch 0011-perf-script-add-time-of-day-option-for-displaying-ev.patch
ApplyPatch 0012-perf-sched-timehist-Use-time-of-day-for-timestamps-i.patch
ApplyPatch 0013-perf-trace-Add-option-to-show-time-of-day-in-output.patch
ApplyPatch 0014-perf-tool-Add-functions-to-specify-time-window-of-in.patch
ApplyPatch 0015-perf-sched-timehist-Add-option-to-specify-time-windo.patch
ApplyPatch 0016-perf-script-Add-option-to-specify-time-window-of-int.patch
ApplyPatch 0017-perf-report-Add-option-to-specify-time-window-of-int.patch
ApplyPatch 0018-Add-module-to-export-perf_clock-to-userspace.patch
ApplyPatch 0019-perf-record-Allow-user-to-control-kernel-user-callch.patch
ApplyPatch 0020-perf-trace-Update-tid-pid-filtering-option-to-levera.patch
ApplyPatch 0021-perf-trace-Add-option-to-display-events-only-for-giv.patch
ApplyPatch 0022-perf-sched-timehist-Add-support-for-hard-and-softirq.patch
ApplyPatch 0023-perf-sched-timehist-Add-support-for-filtering-on-CPU.patch
ApplyPatch 0024-perf-sched-timehist-Cleanup-migrations-handling.patch
ApplyPatch 0025-perf-record-Wakeup-based-on-100k-watermark-WIP.patch
ApplyPatch 0026-perf-sparc-Disable-use-of-kcore.patch
ApplyPatch 0027-perf-sparc-Slow-down-closing-of-fd-s.patch
ApplyPatch 0028-perf-uek2-bus-error-work-round.patch
ApplyPatch 0029-perf-uek2-bus-error-work-round-buffer-read.patch
ApplyPatch 0030-perf-fix_to_show_proper_convergence_stats.patch

%build
# prepare directories
rm -rf $RPM_BUILD_ROOT

cd perf-%{kversion}

%global perf_make \
  make -s -C tools/perf V=1 WERROR=0 NO_LIBUNWIND=1 HAVE_CPLUS_DEMANGLE=1 NO_GTK2=1 NO_STRLCPY=1 NO_BIONIC=1 prefix=%{_prefix}

%{perf_make} DESTDIR=$RPM_BUILD_ROOT all

###
### install
###

%install
[ "$RPM_BUILD_ROOT" != "/" ] && rm -rf $RPM_BUILD_ROOT
cd perf-%{kversion}
%{perf_make} DESTDIR=$RPM_BUILD_ROOT lib=%{_lib} install man
mkdir -p %{buildroot}/%{_mandir}/man1

# perf does not "build" man pages in the buildroot directory
cp tools/perf/Documentation/perf*.1 %{buildroot}/%{_mandir}/man1

###
### clean
###

%post
# fix up for current kernel rpms bringing in perf
rm -f /usr/sbin/perf

%clean
rm -rf $RPM_BUILD_ROOT

%files -n perf
%defattr(-,root,root)
%{_bindir}/perf
%{_bindir}/trace
%dir %{_libdir}/traceevent/plugins
%{_libdir}/traceevent/plugins/*
%dir %{_libexecdir}/perf-core
%{_libexecdir}/perf-core/*
%{_mandir}/man[1-8]/perf*
%{_sysconfdir}/bash_completion.d/perf
%doc perf-%{kversion}/tools/perf/Documentation/examples.txt

%changelog
* Wed Dec 2 2015 Babu Moger <babu.moger@oracle.com> - 4.1.3-rc6
- Fix orabug 22175225

* Fri Jun 5 2015 David Ahern <dsahern@gmail.com> - 4.1.0-rc6
- Initial version
